@@ -51,8 +51,7 @@ impl BufferListRef {
         unsafe { from_glib_full(ffi::gst_buffer_list_copy_deep(self.as_ptr())) }
     }
 
-    #[doc(alias = "gst_buffer_list_remove")]
-    pub fn remove(&mut self, range: impl RangeBounds<usize>) {
+    fn buffer_range_to_start_end_idx(&self, range: impl RangeBounds<usize>) -> (usize, usize) {
         let n = self.len();
         debug_assert!(n <= u32::MAX as usize);
 
@@ -70,6 +69,13 @@ impl BufferListRef {
         };
         assert!(end_idx <= n);
 
+        (start_idx, end_idx)
+    }
+
+    #[doc(alias = "gst_buffer_list_remove")]
+    pub fn remove(&mut self, range: impl RangeBounds<usize>) {
+        let (start_idx, end_idx) = self.buffer_range_to_start_end_idx(range);
+
         unsafe {
             ffi::gst_buffer_list_remove(
                 self.as_mut_ptr(),
@@ -127,16 +133,16 @@ impl BufferListRef {
         self.len() == 0
     }
 
-    pub fn iter(&self) -> Iter {
+    pub fn iter(&self) -> Iter<'_> {
         Iter::new(self)
     }
 
-    pub fn iter_owned(&self) -> IterOwned {
+    pub fn iter_owned(&self) -> IterOwned<'_> {
         IterOwned::new(self)
     }
 
     #[doc(alias = "gst_buffer_list_foreach")]
-    pub fn foreach<F: FnMut(&Buffer, usize) -> ControlFlow<(), ()>>(&self, func: F) -> bool {
+    pub fn foreach<F: FnMut(&Buffer, usize) -> ControlFlow<(), ()>>(&self, func: F) {
         unsafe extern "C" fn trampoline<F: FnMut(&Buffer, usize) -> ControlFlow<(), ()>>(
             buffer: *mut *mut ffi::GstBuffer,
             idx: u32,
@@ -152,11 +158,11 @@ impl BufferListRef {
             let mut func = func;
             let func_ptr: &mut F = &mut func;
 
-            from_glib(ffi::gst_buffer_list_foreach(
+            let _ = ffi::gst_buffer_list_foreach(
                 self.as_ptr() as *mut _,
                 Some(trampoline::<F>),
                 func_ptr as *mut _ as *mut _,
-            ))
+            );
         }
     }
 
@@ -164,7 +170,7 @@ impl BufferListRef {
     pub fn foreach_mut<F: FnMut(Buffer, usize) -> ControlFlow<Option<Buffer>, Option<Buffer>>>(
         &mut self,
         func: F,
-    ) -> bool {
+    ) {
         unsafe extern "C" fn trampoline<
             F: FnMut(Buffer, usize) -> ControlFlow<Option<Buffer>, Option<Buffer>>,
         >(
@@ -202,11 +208,20 @@ impl BufferListRef {
             let mut func = func;
             let func_ptr: &mut F = &mut func;
 
-            from_glib(ffi::gst_buffer_list_foreach(
+            let _ = ffi::gst_buffer_list_foreach(
                 self.as_ptr() as *mut _,
                 Some(trampoline::<F>),
                 func_ptr as *mut _ as *mut _,
-            ))
+            );
+        }
+    }
+
+    pub fn drain(&mut self, range: impl RangeBounds<usize>) -> Drain<'_> {
+        let (start_idx, end_idx) = self.buffer_range_to_start_end_idx(range);
+        Drain {
+            list: self,
+            start_idx,
+            end_idx,
         }
     }
 }
@@ -245,105 +260,189 @@ impl fmt::Debug for BufferListRef {
 
 macro_rules! define_iter(
     ($name:ident, $styp:ty, $get_item:expr) => {
-    #[derive(Debug)]
-    pub struct $name<'a> {
-        list: &'a BufferListRef,
-        idx: usize,
-        size: usize,
-    }
-
-    impl<'a> $name<'a> {
-        fn new(list: &'a BufferListRef) -> $name<'a> {
-            skip_assert_initialized!();
-            $name {
-                list,
-                idx: 0,
-                size: list.len(),
-            }
-        }
-    }
-
-    #[allow(clippy::redundant_closure_call)]
-    impl<'a> Iterator for $name<'a> {
-        type Item = $styp;
-
-        fn next(&mut self) -> Option<Self::Item> {
-            if self.idx >= self.size {
-                return None;
-            }
-
-            let item = $get_item(self.list, self.idx).unwrap();
-            self.idx += 1;
-
-            Some(item)
-        }
-
-        fn size_hint(&self) -> (usize, Option<usize>) {
-            let remaining = self.size - self.idx;
-
-            (remaining, Some(remaining))
-        }
-
-        fn count(self) -> usize {
-            self.size - self.idx
-        }
-
-        fn nth(&mut self, n: usize) -> Option<Self::Item> {
-            let (end, overflow) = self.idx.overflowing_add(n);
-            if end >= self.size || overflow {
-                self.idx = self.size;
-                None
-            } else {
-                self.idx = end + 1;
-                Some($get_item(self.list, end).unwrap())
-            }
-        }
-
-        fn last(self) -> Option<Self::Item> {
-            if self.idx == self.size {
-                None
-            } else {
-                Some($get_item(self.list, self.size - 1).unwrap())
-            }
-        }
-    }
-
-    #[allow(clippy::redundant_closure_call)]
-    impl<'a> DoubleEndedIterator for $name<'a> {
-        fn next_back(&mut self) -> Option<Self::Item> {
-            if self.idx == self.size {
-                return None;
-            }
-
-            self.size -= 1;
-            Some($get_item(self.list, self.size).unwrap())
-        }
-
-        fn nth_back(&mut self, n: usize) -> Option<Self::Item> {
-            let (end, overflow) = self.size.overflowing_sub(n);
-            if end <= self.idx || overflow {
-                self.idx = self.size;
-                None
-            } else {
-                self.size = end - 1;
-                Some($get_item(self.list, self.size).unwrap())
-            }
-        }
-    }
-
-    impl<'a> ExactSizeIterator for $name<'a> {}
-    impl<'a> std::iter::FusedIterator for $name<'a> {}
+        crate::utils::define_fixed_size_iter!(
+            $name, &'a BufferListRef, $styp,
+            |collection: &BufferListRef| collection.len(),
+            $get_item
+        );
     }
 );
 
-define_iter!(Iter, &'a BufferRef, |list: &'a BufferListRef, idx| {
-    list.get(idx)
+define_iter!(Iter, &'a BufferRef, |list: &BufferListRef, idx| unsafe {
+    let ptr = ffi::gst_buffer_list_get(list.as_mut_ptr(), idx as u32);
+    BufferRef::from_ptr(ptr)
 });
 
-define_iter!(IterOwned, Buffer, |list: &BufferListRef, idx| {
-    list.get_owned(idx)
+define_iter!(IterOwned, Buffer, |list: &BufferListRef, idx| unsafe {
+    let ptr = ffi::gst_buffer_list_get(list.as_mut_ptr(), idx as u32);
+    from_glib_none(ptr)
 });
 
+#[derive(Debug)]
+pub struct Drain<'a> {
+    list: &'a mut BufferListRef,
+    start_idx: usize,
+    end_idx: usize,
+}
+
+impl Iterator for Drain<'_> {
+    type Item = Buffer;
+
+    #[inline]
+    fn next(&mut self) -> Option<Self::Item> {
+        if self.start_idx >= self.end_idx {
+            return None;
+        }
+
+        let buffer = unsafe {
+            let buffer = Buffer::from_glib_none(ffi::gst_buffer_list_get(
+                self.list.as_mut_ptr(),
+                self.start_idx as u32,
+            ));
+            ffi::gst_buffer_list_remove(self.list.as_mut_ptr(), self.start_idx as u32, 1);
+            buffer
+        };
+
+        self.end_idx -= 1;
+
+        Some(buffer)
+    }
+
+    #[inline]
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        let remaining = self.end_idx - self.start_idx;
+
+        (remaining, Some(remaining))
+    }
+
+    #[inline]
+    fn count(self) -> usize {
+        self.end_idx - self.start_idx
+    }
+
+    #[inline]
+    fn nth(&mut self, n: usize) -> Option<Self::Item> {
+        let (end, overflow) = self.start_idx.overflowing_add(n);
+        if end >= self.end_idx || overflow {
+            unsafe {
+                ffi::gst_buffer_list_remove(
+                    self.list.as_mut_ptr(),
+                    self.start_idx as u32,
+                    (self.end_idx - self.start_idx) as u32,
+                );
+            }
+            self.start_idx = self.end_idx;
+            None
+        } else {
+            let buffer = unsafe {
+                let buffer = Buffer::from_glib_none(ffi::gst_buffer_list_get(
+                    self.list.as_mut_ptr(),
+                    end as u32,
+                ));
+                ffi::gst_buffer_list_remove(
+                    self.list.as_mut_ptr(),
+                    self.start_idx as u32,
+                    n as u32,
+                );
+                buffer
+            };
+            self.end_idx -= n;
+            Some(buffer)
+        }
+    }
+
+    #[inline]
+    fn last(mut self) -> Option<Self::Item> {
+        if self.start_idx == self.end_idx {
+            None
+        } else {
+            let buffer = unsafe {
+                let buffer = Buffer::from_glib_none(ffi::gst_buffer_list_get(
+                    self.list.as_mut_ptr(),
+                    self.end_idx as u32 - 1,
+                ));
+                ffi::gst_buffer_list_remove(
+                    self.list.as_mut_ptr(),
+                    self.start_idx as u32,
+                    (self.end_idx - self.start_idx) as u32,
+                );
+                buffer
+            };
+            self.end_idx = self.start_idx;
+            Some(buffer)
+        }
+    }
+}
+
+impl DoubleEndedIterator for Drain<'_> {
+    #[inline]
+    fn next_back(&mut self) -> Option<Self::Item> {
+        if self.start_idx == self.end_idx {
+            return None;
+        }
+
+        self.end_idx -= 1;
+        let buffer = unsafe {
+            let buffer = Buffer::from_glib_none(ffi::gst_buffer_list_get(
+                self.list.as_mut_ptr(),
+                self.end_idx as u32,
+            ));
+            ffi::gst_buffer_list_remove(self.list.as_mut_ptr(), self.end_idx as u32, 1);
+            buffer
+        };
+
+        Some(buffer)
+    }
+
+    #[inline]
+    fn nth_back(&mut self, n: usize) -> Option<Self::Item> {
+        let (end, overflow) = self.end_idx.overflowing_sub(n);
+        if end <= self.start_idx || overflow {
+            unsafe {
+                ffi::gst_buffer_list_remove(
+                    self.list.as_mut_ptr(),
+                    self.start_idx as u32,
+                    (self.end_idx - self.start_idx) as u32,
+                );
+            }
+            self.start_idx = self.end_idx;
+            None
+        } else {
+            self.end_idx = end - 1;
+            let buffer = unsafe {
+                let buffer = Buffer::from_glib_none(ffi::gst_buffer_list_get(
+                    self.list.as_mut_ptr(),
+                    self.end_idx as u32,
+                ));
+                ffi::gst_buffer_list_remove(self.list.as_mut_ptr(), self.end_idx as u32, n as u32);
+                buffer
+            };
+
+            Some(buffer)
+        }
+    }
+}
+
+impl ExactSizeIterator for Drain<'_> {}
+
+impl std::iter::FusedIterator for Drain<'_> {}
+
+impl Drop for Drain<'_> {
+    fn drop(&mut self) {
+        if self.start_idx >= self.end_idx {
+            return;
+        }
+
+        unsafe {
+            ffi::gst_buffer_list_remove(
+                self.list.as_mut_ptr(),
+                self.start_idx as u32,
+                (self.end_idx - self.start_idx) as u32,
+            );
+        }
+    }
+}
+
 impl<'a> IntoIterator for &'a BufferListRef {
     type IntoIter = Iter<'a>;
     type Item = &'a BufferRef;
@@ -549,4 +648,104 @@ mod tests {
 
         assert!(buffer_list.is_empty());
     }
+
+    #[test]
+    fn test_drain() {
+        crate::init().unwrap();
+
+        let mut buffer_list = make_buffer_list(10);
+
+        let buffers_removed = buffer_list
+            .make_mut()
+            .drain(0..2)
+            .map(|buf| buf.pts().unwrap() / ClockTime::SECOND)
+            .collect::<Vec<_>>();
+
+        assert_eq!(buffers_removed, &[0, 1]);
+
+        let buffers_left = buffer_list
+            .iter()
+            .map(|buf| buf.pts().unwrap() / ClockTime::SECOND)
+            .collect::<Vec<_>>();
+
+        assert_eq!(buffers_left, &[2, 3, 4, 5, 6, 7, 8, 9]);
+
+        let buffers_removed = buffer_list
+            .make_mut()
+            .drain(0..=2)
+            .map(|buf| buf.pts().unwrap() / ClockTime::SECOND)
+            .collect::<Vec<_>>();
+
+        assert_eq!(buffers_removed, &[2, 3, 4]);
+
+        let buffers_left = buffer_list
+            .iter()
+            .map(|buf| buf.pts().unwrap() / ClockTime::SECOND)
+            .collect::<Vec<_>>();
+
+        assert_eq!(buffers_left, &[5, 6, 7, 8, 9]);
+
+        let buffers_removed = buffer_list
+            .make_mut()
+            .drain(2..)
+            .map(|buf| buf.pts().unwrap() / ClockTime::SECOND)
+            .collect::<Vec<_>>();
+
+        assert_eq!(buffers_removed, &[7, 8, 9]);
+
+        let buffers_left = buffer_list
+            .iter()
+            .map(|buf| buf.pts().unwrap() / ClockTime::SECOND)
+            .collect::<Vec<_>>();
+
+        assert_eq!(buffers_left, &[5, 6]);
+
+        let buffers_removed = buffer_list
+            .make_mut()
+            .drain(..)
+            .map(|buf| buf.pts().unwrap() / ClockTime::SECOND)
+            .collect::<Vec<_>>();
+
+        assert_eq!(buffers_removed, &[5, 6]);
+
+        assert!(buffer_list.is_empty());
+    }
+
+    #[test]
+    fn test_drain_drop() {
+        crate::init().unwrap();
+
+        let mut buffer_list = make_buffer_list(10);
+
+        buffer_list.make_mut().drain(0..2);
+
+        let buffers_left = buffer_list
+            .iter()
+            .map(|buf| buf.pts().unwrap() / ClockTime::SECOND)
+            .collect::<Vec<_>>();
+
+        assert_eq!(buffers_left, &[2, 3, 4, 5, 6, 7, 8, 9]);
+
+        buffer_list.make_mut().drain(0..=2);
+
+        let buffers_left = buffer_list
+            .iter()
+            .map(|buf| buf.pts().unwrap() / ClockTime::SECOND)
+            .collect::<Vec<_>>();
+
+        assert_eq!(buffers_left, &[5, 6, 7, 8, 9]);
+
+        buffer_list.make_mut().drain(2..);
+
+        let buffers_left = buffer_list
+            .iter()
+            .map(|buf| buf.pts().unwrap() / ClockTime::SECOND)
+            .collect::<Vec<_>>();
+
+        assert_eq!(buffers_left, &[5, 6]);
+
+        buffer_list.make_mut().drain(..);
+
+        assert!(buffer_list.is_empty());
+    }
 }
