@@ -1,6 +1,6 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use std::{ffi::CStr, future::Future, mem, num::NonZeroU64, pin::Pin};
+use std::{ffi::CStr, future::Future, mem, num::NonZeroU64, ops::ControlFlow, pin::Pin};
 
 use glib::translate::*;
 use itertools::Itertools;
@@ -18,8 +18,8 @@ use crate::{
 
 impl Element {
     #[doc(alias = "gst_element_link_many")]
-    pub fn link_many(
-        elements: impl IntoIterator<Item = impl AsRef<Element> + Clone>,
+    pub fn link_many<E: AsRef<Element> + Clone>(
+        elements: impl IntoIterator<Item = E>,
     ) -> Result<(), glib::BoolError> {
         skip_assert_initialized!();
         for (src, dest) in elements.into_iter().tuple_windows() {
@@ -40,7 +40,7 @@ impl Element {
     }
 
     #[doc(alias = "gst_element_unlink_many")]
-    pub fn unlink_many(elements: impl IntoIterator<Item = impl AsRef<Element> + Clone>) {
+    pub fn unlink_many<E: AsRef<Element> + Clone>(elements: impl IntoIterator<Item = E>) {
         skip_assert_initialized!();
         for (src, dest) in elements.into_iter().tuple_windows() {
             unsafe {
@@ -102,12 +102,7 @@ impl FromGlib<libc::c_ulong> for NotifyWatchId {
     }
 }
 
-mod sealed {
-    pub trait Sealed {}
-    impl<T: super::IsA<super::Element>> Sealed for T {}
-}
-
-pub trait ElementExtManual: sealed::Sealed + IsA<Element> + 'static {
+pub trait ElementExtManual: IsA<Element> + 'static {
     #[doc(alias = "get_element_class")]
     #[inline]
     fn element_class(&self) -> &glib::Class<Element> {
@@ -338,6 +333,81 @@ pub trait ElementExtManual: sealed::Sealed + IsA<Element> + 'static {
         }
     }
 
+    #[doc(alias = "gst_element_foreach_pad")]
+    fn foreach_pad<F: FnMut(&Element, &Pad) -> ControlFlow<()>>(&self, func: F) {
+        unsafe extern "C" fn trampoline<F: FnMut(&Element, &Pad) -> ControlFlow<()>>(
+            element: *mut ffi::GstElement,
+            pad: *mut ffi::GstPad,
+            user_data: glib::ffi::gpointer,
+        ) -> glib::ffi::gboolean {
+            let element = from_glib_borrow(element);
+            let pad = from_glib_borrow(pad);
+            let callback = user_data as *mut F;
+            (*callback)(&element, &pad).is_continue().into_glib()
+        }
+
+        unsafe {
+            let mut func = func;
+            let func_ptr: &mut F = &mut func;
+
+            let _ = ffi::gst_element_foreach_pad(
+                self.as_ptr() as *mut _,
+                Some(trampoline::<F>),
+                func_ptr as *mut _ as *mut _,
+            );
+        }
+    }
+
+    #[doc(alias = "gst_element_foreach_sink_pad")]
+    fn foreach_sink_pad<F: FnMut(&Element, &Pad) -> ControlFlow<()>>(&self, func: F) {
+        unsafe extern "C" fn trampoline<P: FnMut(&Element, &Pad) -> ControlFlow<()>>(
+            element: *mut ffi::GstElement,
+            pad: *mut ffi::GstPad,
+            user_data: glib::ffi::gpointer,
+        ) -> glib::ffi::gboolean {
+            let element = from_glib_borrow(element);
+            let pad = from_glib_borrow(pad);
+            let callback = user_data as *mut P;
+            (*callback)(&element, &pad).is_continue().into_glib()
+        }
+
+        unsafe {
+            let mut func = func;
+            let func_ptr: &mut F = &mut func;
+
+            let _ = ffi::gst_element_foreach_sink_pad(
+                self.as_ptr() as *mut _,
+                Some(trampoline::<F>),
+                func_ptr as *mut _ as *mut _,
+            );
+        }
+    }
+
+    #[doc(alias = "gst_element_foreach_src_pad")]
+    fn foreach_src_pad<F: FnMut(&Element, &Pad) -> ControlFlow<()>>(&self, func: F) {
+        unsafe extern "C" fn trampoline<P: FnMut(&Element, &Pad) -> ControlFlow<()>>(
+            element: *mut ffi::GstElement,
+            pad: *mut ffi::GstPad,
+            user_data: glib::ffi::gpointer,
+        ) -> glib::ffi::gboolean {
+            let element = from_glib_borrow(element);
+            let pad = from_glib_borrow(pad);
+            let callback = user_data as *mut P;
+            (*callback)(&element, &pad).is_continue().into_glib()
+        }
+
+        unsafe {
+            let mut func = func;
+            let func_ptr: &mut F = &mut func;
+
+            let _ = ffi::gst_element_foreach_src_pad(
+                self.as_ptr() as *mut _,
+                Some(trampoline::<F>),
+                func_ptr as *mut _ as *mut _,
+            );
+        }
+    }
+
     fn num_pads(&self) -> u16 {
         unsafe {
             let elt: &ffi::GstElement = &*(self.as_ptr() as *const _);
@@ -631,11 +701,7 @@ pub trait ElementExtManual: sealed::Sealed + IsA<Element> + 'static {
     #[doc(alias = "get_current_clock_time")]
     #[doc(alias = "gst_element_get_current_clock_time")]
     fn current_clock_time(&self) -> Option<crate::ClockTime> {
-        if let Some(clock) = self.clock() {
-            clock.time()
-        } else {
-            crate::ClockTime::NONE
-        }
+        self.clock().as_ref().map(crate::Clock::time)
     }
 
     #[doc(alias = "gst_element_get_request_pad")]
@@ -1247,10 +1313,26 @@ mod tests {
         identity.foreach_pad(|_element, pad| {
             pad_names.push(pad.name());
 
-            true
+            ControlFlow::Continue(())
         });
         pad_names.sort();
         assert_eq!(pad_names, vec![String::from("sink"), String::from("src")]);
+
+        pad_names.clear();
+        identity.foreach_sink_pad(|_element, pad| {
+            pad_names.push(pad.name());
+
+            ControlFlow::Continue(())
+        });
+        assert_eq!(pad_names, vec![String::from("sink")]);
+
+        pad_names.clear();
+        identity.foreach_src_pad(|_element, pad| {
+            pad_names.push(pad.name());
+
+            ControlFlow::Continue(())
+        });
+        assert_eq!(pad_names, vec![String::from("src")]);
     }
 
     #[test]
