@@ -1,18 +1,69 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use glib::translate::*;
-use glib::StaticType;
-
-use std::fmt;
-use std::io::{self, prelude::*};
-use std::time::Duration;
-
-use super::{Format, FormattedValueError, GenericFormattedValue, Signed};
-use super::{
-    FormattedValue, FormattedValueFullRange, FormattedValueIntrinsic, FormattedValueNoneBuilder,
-    SpecificFormattedValue, SpecificFormattedValueFullRange, SpecificFormattedValueIntrinsic,
+use std::{
+    fmt,
+    io::{self, prelude::*},
+    time::Duration,
 };
 
+use crate::{ffi, prelude::*};
+use glib::translate::*;
+
+use super::{
+    Format, FormattedValue, FormattedValueError, FormattedValueFullRange, FormattedValueIntrinsic,
+    FormattedValueNoneBuilder, GenericFormattedValue, Signed, SpecificFormattedValue,
+    SpecificFormattedValueFullRange, SpecificFormattedValueIntrinsic,
+};
+
+const TRY_FROM_FLOAT_SECS_ERROR_MSG: &str =
+    "can not convert float seconds to ClockTime: value is either negative, too big or NaN";
+
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub struct TryFromFloatSecsError;
+
+impl fmt::Display for TryFromFloatSecsError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.write_str(TRY_FROM_FLOAT_SECS_ERROR_MSG)
+    }
+}
+
+impl std::error::Error for TryFromFloatSecsError {}
+
+// rustdoc-stripper-ignore-next
+/// A Time quantity
+///
+/// Some functions enforce format specific quantities. This type can be used when
+/// a Duration or an Instant is expected. It comes with functions to perform computations without the
+/// need to retrieve the inner integer.
+///
+/// # Examples
+///
+/// ```rust
+/// # use gstreamer::{prelude::*, ClockTime};
+/// // Regular constructors (can be used in `const` contexts)
+/// const FORTY_TWO_NS: ClockTime = ClockTime::from_nseconds(42);
+/// const TWO_US: ClockTime = ClockTime::from_useconds(2);
+/// let three_ms: ClockTime = ClockTime::from_mseconds(3);
+/// let four_s: ClockTime = ClockTime::from_seconds(4);
+///
+/// // Convenience constructors (not `const`)
+/// let forty_two_ns = 42.nseconds();
+/// let two_us = 2.useconds();
+/// let three_ms = 3.mseconds();
+/// let four_s = 4.seconds();
+///
+/// // All four arithmetic operations
+/// let deadline = (2.mseconds() + 512.useconds()) * 2 / 3;
+///
+/// // Comparisons
+/// if deadline > ClockTime::MSECOND {
+///     println!("Greater");
+/// }
+/// ```
+///
+/// See [the documentation of the `format` module] for more examples.
+///
+/// [the documentation of the `format` module]: ./index.html
 #[derive(PartialEq, Eq, PartialOrd, Ord, Hash, Clone, Copy, Default)]
 pub struct ClockTime(u64);
 
@@ -28,26 +79,42 @@ impl ClockTime {
     // checker-ignore-item
     pub const MAX: ClockTime = ClockTime(ffi::GST_CLOCK_TIME_NONE - 1);
 
+    #[inline]
     pub const fn hours(self) -> u64 {
         self.0 / Self::SECOND.0 / 60 / 60
     }
 
+    #[inline]
     pub const fn minutes(self) -> u64 {
         self.0 / Self::SECOND.0 / 60
     }
 
+    #[inline]
     pub const fn seconds(self) -> u64 {
         self.0 / Self::SECOND.0
     }
 
+    #[inline]
+    pub fn seconds_f32(self) -> f32 {
+        self.0 as f32 / Self::SECOND.0 as f32
+    }
+
+    #[inline]
+    pub fn seconds_f64(self) -> f64 {
+        self.0 as f64 / Self::SECOND.0 as f64
+    }
+
+    #[inline]
     pub const fn mseconds(self) -> u64 {
         self.0 / Self::MSECOND.0
     }
 
+    #[inline]
     pub const fn useconds(self) -> u64 {
         self.0 / Self::USECOND.0
     }
 
+    #[inline]
     pub const fn nseconds(self) -> u64 {
         self.0
     }
@@ -59,6 +126,7 @@ impl ClockTime {
     ///
     /// Panics if the resulting duration in nanoseconds exceeds the `u64` range.
     #[track_caller]
+    #[inline]
     pub const fn from_seconds(seconds: u64) -> Self {
         skip_assert_initialized!();
         // `Option::expect` is not `const` as of rustc 1.63.0.
@@ -68,6 +136,62 @@ impl ClockTime {
         })
     }
 
+    // rustdoc-stripper-ignore-next
+    /// Builds a new `ClockTime` which value is the given number of seconds.
+    ///
+    /// Returns an error if seconds is negative, infinite or NaN, or
+    /// the resulting duration in nanoseconds exceeds the `u64` range.
+    #[inline]
+    pub fn try_from_seconds_f32(seconds: f32) -> Result<Self, TryFromFloatSecsError> {
+        skip_assert_initialized!();
+
+        let dur = Duration::try_from_secs_f32(seconds).map_err(|_| TryFromFloatSecsError)?;
+        ClockTime::try_from(dur).map_err(|_| TryFromFloatSecsError)
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Builds a new `ClockTime` which value is the given number of seconds.
+    ///
+    /// # Panics
+    ///
+    /// Panics if seconds is negative, infinite or NaN, or the resulting duration
+    /// in nanoseconds exceeds the `u64` range.
+    #[track_caller]
+    #[inline]
+    pub fn from_seconds_f32(seconds: f32) -> Self {
+        skip_assert_initialized!();
+
+        Self::try_from_seconds_f32(seconds).expect(TRY_FROM_FLOAT_SECS_ERROR_MSG)
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Builds a new `ClockTime` which value is the given number of seconds.
+    ///
+    /// Returns an error if seconds is negative, infinite or NaN, or
+    /// the resulting duration in nanoseconds exceeds the `u64` range.
+    #[inline]
+    pub fn try_from_seconds_f64(seconds: f64) -> Result<Self, TryFromFloatSecsError> {
+        skip_assert_initialized!();
+
+        let dur = Duration::try_from_secs_f64(seconds).map_err(|_| TryFromFloatSecsError)?;
+        ClockTime::try_from(dur).map_err(|_| TryFromFloatSecsError)
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Builds a new `ClockTime` which value is the given number of seconds.
+    ///
+    /// # Panics
+    ///
+    /// Panics if seconds is negative, infinite or NaN, or the resulting duration
+    /// in nanoseconds exceeds the `u64` range.
+    #[track_caller]
+    #[inline]
+    pub fn from_seconds_f64(seconds: f64) -> Self {
+        skip_assert_initialized!();
+
+        Self::try_from_seconds_f64(seconds).expect(TRY_FROM_FLOAT_SECS_ERROR_MSG)
+    }
+
     // rustdoc-stripper-ignore-next
     /// Builds a new `ClockTime` which value is the given number of milliseconds.
     ///
@@ -75,6 +199,7 @@ impl ClockTime {
     ///
     /// Panics if the resulting duration in nanoseconds exceeds the `u64` range.
     #[track_caller]
+    #[inline]
     pub const fn from_mseconds(mseconds: u64) -> Self {
         skip_assert_initialized!();
         // `Option::expect` is not `const` as of rustc 1.63.0.
@@ -91,6 +216,7 @@ impl ClockTime {
     ///
     /// Panics if the resulting duration in nanoseconds exceeds the `u64` range.
     #[track_caller]
+    #[inline]
     pub const fn from_useconds(useconds: u64) -> Self {
         skip_assert_initialized!();
         // `Option::expect` is not `const` as of rustc 1.63.0.
@@ -108,6 +234,7 @@ impl ClockTime {
     /// Panics if the requested duration equals `GST_CLOCK_TIME_NONE`
     /// (`u64::MAX`).
     #[track_caller]
+    #[inline]
     pub const fn from_nseconds(nseconds: u64) -> Self {
         skip_assert_initialized!();
         assert!(
@@ -121,6 +248,7 @@ impl ClockTime {
 impl Signed<ClockTime> {
     // rustdoc-stripper-ignore-next
     /// Returns the `self` in nanoseconds.
+    #[inline]
     pub fn nseconds(self) -> Signed<u64> {
         match self {
             Signed::Positive(val) => Signed::Positive(val.nseconds()),
@@ -130,6 +258,7 @@ impl Signed<ClockTime> {
 
     // rustdoc-stripper-ignore-next
     /// Creates new value from nanoseconds.
+    #[inline]
     pub fn from_nseconds(val: Signed<u64>) -> Self {
         skip_assert_initialized!();
         match val {
@@ -140,6 +269,7 @@ impl Signed<ClockTime> {
 
     // rustdoc-stripper-ignore-next
     /// Returns the `self` in microseconds.
+    #[inline]
     pub fn useconds(self) -> Signed<u64> {
         match self {
             Signed::Positive(val) => Signed::Positive(val.useconds()),
@@ -149,6 +279,7 @@ impl Signed<ClockTime> {
 
     // rustdoc-stripper-ignore-next
     /// Creates new value from microseconds.
+    #[inline]
     pub fn from_useconds(val: Signed<u64>) -> Self {
         skip_assert_initialized!();
         match val {
@@ -159,6 +290,7 @@ impl Signed<ClockTime> {
 
     // rustdoc-stripper-ignore-next
     /// Returns the `self` in milliseconds.
+    #[inline]
     pub fn mseconds(self) -> Signed<u64> {
         match self {
             Signed::Positive(val) => Signed::Positive(val.mseconds()),
@@ -168,6 +300,7 @@ impl Signed<ClockTime> {
 
     // rustdoc-stripper-ignore-next
     /// Creates new value from milliseconds.
+    #[inline]
     pub fn from_mseconds(val: Signed<u64>) -> Self {
         skip_assert_initialized!();
         match val {
@@ -178,6 +311,7 @@ impl Signed<ClockTime> {
 
     // rustdoc-stripper-ignore-next
     /// Returns the `self` in seconds.
+    #[inline]
     pub fn seconds(self) -> Signed<u64> {
         match self {
             Signed::Positive(val) => Signed::Positive(val.seconds()),
@@ -185,8 +319,29 @@ impl Signed<ClockTime> {
         }
     }
 
+    // rustdoc-stripper-ignore-next
+    /// Returns the `self` in f32 seconds.
+    #[inline]
+    pub fn seconds_f32(self) -> f32 {
+        match self {
+            Signed::Positive(val) => val.seconds_f32(),
+            Signed::Negative(val) => -val.seconds_f32(),
+        }
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Returns the `self` in f64 seconds.
+    #[inline]
+    pub fn seconds_f64(self) -> f64 {
+        match self {
+            Signed::Positive(val) => val.seconds_f64(),
+            Signed::Negative(val) => -val.seconds_f64(),
+        }
+    }
+
     // rustdoc-stripper-ignore-next
     /// Creates new value from seconds.
+    #[inline]
     pub fn from_seconds(val: Signed<u64>) -> Self {
         skip_assert_initialized!();
         match val {
@@ -194,6 +349,72 @@ impl Signed<ClockTime> {
             Signed::Negative(val) => Signed::Negative(ClockTime::from_seconds(val)),
         }
     }
+
+    // rustdoc-stripper-ignore-next
+    /// Builds a new `Signed<ClockTime>` which value is the given number of seconds.
+    ///
+    /// Returns an error if seconds is infinite or NaN, or
+    /// the resulting duration in nanoseconds exceeds the `u64` range.
+    #[inline]
+    pub fn try_from_seconds_f32(seconds: f32) -> Result<Self, TryFromFloatSecsError> {
+        skip_assert_initialized!();
+
+        ClockTime::try_from_seconds_f32(seconds.abs()).map(|ct| {
+            if seconds.is_sign_positive() {
+                Signed::Positive(ct)
+            } else {
+                Signed::Negative(ct)
+            }
+        })
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Builds a new `Signed<ClockTime>` which value is the given number of seconds.
+    ///
+    /// # Panics
+    ///
+    /// Panics if seconds is infinite or NaN, or the resulting duration
+    /// in nanoseconds exceeds the `u64` range.
+    #[track_caller]
+    #[inline]
+    pub fn from_seconds_f32(seconds: f32) -> Self {
+        skip_assert_initialized!();
+
+        Self::try_from_seconds_f32(seconds).expect(TRY_FROM_FLOAT_SECS_ERROR_MSG)
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Builds a new `Signed<ClockTime>` which value is the given number of seconds.
+    ///
+    /// Returns an error if seconds is infinite or NaN, or
+    /// the resulting duration in nanoseconds exceeds the `u64` range.
+    #[inline]
+    pub fn try_from_seconds_f64(seconds: f64) -> Result<Self, TryFromFloatSecsError> {
+        skip_assert_initialized!();
+
+        ClockTime::try_from_seconds_f64(seconds.abs()).map(|ct| {
+            if seconds.is_sign_positive() {
+                Signed::Positive(ct)
+            } else {
+                Signed::Negative(ct)
+            }
+        })
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Builds a new `Signed<ClockTime>` which value is the given number of seconds.
+    ///
+    /// # Panics
+    ///
+    /// Panics if seconds is infinite or NaN, or the resulting duration
+    /// in nanoseconds exceeds the `u64` range.
+    #[track_caller]
+    #[inline]
+    pub fn from_seconds_f64(seconds: f64) -> Self {
+        skip_assert_initialized!();
+
+        Self::try_from_seconds_f64(seconds).expect(TRY_FROM_FLOAT_SECS_ERROR_MSG)
+    }
 }
 
 impl_format_value_traits!(ClockTime, Time, Time, u64);
@@ -230,31 +451,37 @@ pub trait TimeFormatConstructor {
 
 impl TimeFormatConstructor for u64 {
     #[track_caller]
+    #[inline]
     fn nseconds(self) -> ClockTime {
         ClockTime::from_nseconds(self)
     }
 
     #[track_caller]
+    #[inline]
     fn useconds(self) -> ClockTime {
         ClockTime::from_useconds(self)
     }
 
     #[track_caller]
+    #[inline]
     fn mseconds(self) -> ClockTime {
         ClockTime::from_mseconds(self)
     }
 
     #[track_caller]
+    #[inline]
     fn seconds(self) -> ClockTime {
         ClockTime::from_seconds(self)
     }
 
     #[track_caller]
+    #[inline]
     fn minutes(self) -> ClockTime {
         ClockTime::from_seconds(self * 60)
     }
 
     #[track_caller]
+    #[inline]
     fn hours(self) -> ClockTime {
         ClockTime::from_seconds(self * 60 * 60)
     }
@@ -269,6 +496,7 @@ pub enum ClockTimeValueTypeOrNoneChecker {}
 unsafe impl glib::value::ValueTypeChecker for ClockTimeValueTypeOrNoneChecker {
     type Error = glib::value::ValueTypeMismatchOrNoneError<glib::value::ValueTypeMismatchError>;
 
+    #[inline]
     fn check(value: &glib::Value) -> Result<(), Self::Error> {
         skip_assert_initialized!();
         glib::value::GenericValueTypeChecker::<ClockTime>::check(value)?;
@@ -282,9 +510,10 @@ unsafe impl glib::value::ValueTypeChecker for ClockTimeValueTypeOrNoneChecker {
     }
 }
 
-unsafe impl<'a> glib::value::FromValue<'a> for ClockTime {
+unsafe impl glib::value::FromValue<'_> for ClockTime {
     type Checker = ClockTimeValueTypeOrNoneChecker;
 
+    #[inline]
     unsafe fn from_value(value: &glib::Value) -> ClockTime {
         skip_assert_initialized!();
         ClockTime(glib::gobject_ffi::g_value_get_uint64(
@@ -294,6 +523,7 @@ unsafe impl<'a> glib::value::FromValue<'a> for ClockTime {
 }
 
 impl glib::value::ToValue for ClockTime {
+    #[inline]
     fn to_value(&self) -> glib::Value {
         let mut value = glib::Value::for_value_type::<ClockTime>();
         let gct = self.into_glib();
@@ -307,12 +537,14 @@ impl glib::value::ToValue for ClockTime {
         value
     }
 
+    #[inline]
     fn value_type(&self) -> glib::Type {
         Self::static_type()
     }
 }
 
 impl glib::value::ToValueOptional for ClockTime {
+    #[inline]
     fn to_value_optional(opt: Option<&Self>) -> glib::Value {
         skip_assert_initialized!();
         let mut value = glib::Value::for_value_type::<ClockTime>();
@@ -323,10 +555,28 @@ impl glib::value::ToValueOptional for ClockTime {
     }
 }
 
+impl From<ClockTime> for glib::Value {
+    #[inline]
+    fn from(v: ClockTime) -> glib::Value {
+        glib::value::ToValue::to_value(&v)
+    }
+}
+
 #[doc(hidden)]
-impl glib::StaticType for ClockTime {
+impl StaticType for ClockTime {
+    #[inline]
     fn static_type() -> glib::Type {
-        <u64 as glib::StaticType>::static_type()
+        <u64 as StaticType>::static_type()
+    }
+}
+
+impl HasParamSpec for ClockTime {
+    type ParamSpec = glib::ParamSpecUInt64;
+    type SetValue = Self;
+    type BuilderFn = fn(&str) -> glib::ParamSpecUInt64Builder;
+
+    fn param_spec_builder() -> Self::BuilderFn {
+        Self::ParamSpec::builder
     }
 }
 
@@ -344,13 +594,14 @@ impl std::error::Error for DurationError {}
 impl TryFrom<Duration> for ClockTime {
     type Error = DurationError;
 
+    #[inline]
     fn try_from(d: Duration) -> Result<Self, Self::Error> {
         skip_assert_initialized!();
 
         let nanos = d.as_nanos();
 
-        // Note: `std::u64::MAX` is `ClockTime::NONE`.
-        if nanos >= std::u64::MAX as u128 {
+        // Note: `u64::MAX` is `ClockTime::NONE`.
+        if nanos >= u64::MAX as u128 {
             return Err(DurationError);
         }
 
@@ -359,6 +610,7 @@ impl TryFrom<Duration> for ClockTime {
 }
 
 impl From<ClockTime> for Duration {
+    #[inline]
     fn from(t: ClockTime) -> Self {
         skip_assert_initialized!();
 
@@ -368,6 +620,7 @@ impl From<ClockTime> for Duration {
 
 impl_common_ops_for_newtype_uint!(ClockTime, u64);
 impl_signed_div_mul!(ClockTime, u64);
+impl_signed_int_into_signed!(ClockTime, u64);
 
 // rustdoc-stripper-ignore-next
 /// Tell [`pad_clocktime`] what kind of time we're formatting
@@ -398,9 +651,10 @@ enum Sign {
 /// contain the sign; that will be added by this method.
 fn pad_clocktime(f: &mut fmt::Formatter<'_>, sign: Sign, buf: &str) -> fmt::Result {
     skip_assert_initialized!();
-    use self::Sign::*;
     use std::fmt::{Alignment, Write};
 
+    use self::Sign::*;
+
     // Start by determining how we're padding, gathering
     // settings from the Formatter and the Sign
 
@@ -437,7 +691,7 @@ fn pad_clocktime(f: &mut fmt::Formatter<'_>, sign: Sign, buf: &str) -> fmt::Resu
 
         // Align center: Split equally between left and right side
         // If the required padding is odd, the right side gets one more char
-        Alignment::Center => (padding / 2, 0, (padding + 1) / 2),
+        Alignment::Center => (padding / 2, 0, padding.div_ceil(2)),
     };
 
     // And now for the actual writing
@@ -476,17 +730,17 @@ fn write_clocktime<W: io::Write>(
         let (h, m) = num_integer::div_rem(m, 60);
 
         // Write HH:MM:SS
-        write!(writer, "{}:{:02}:{:02}", h, m, s)?;
+        write!(writer, "{h}:{m:02}:{s:02}")?;
 
         if precision > 0 {
             // Format the nanoseconds into a stack-allocated string
             // The value is zero-padded so always 9 digits long
             let mut buf = [0u8; 9];
-            write!(&mut buf[..], "{:09}", ns).unwrap();
+            write!(&mut buf[..], "{ns:09}").unwrap();
             let buf_str = std::str::from_utf8(&buf[..]).unwrap();
 
             // Write decimal point and a prefix of the nanoseconds for more precision
-            write!(writer, ".{:.p$}", buf_str, p = precision)?;
+            write!(writer, ".{buf_str:.precision$}")?;
         }
     } else {
         // Undefined time
@@ -577,9 +831,10 @@ impl std::iter::Sum for ClockTime {
 
 #[cfg(test)]
 mod tests {
+    use opt_ops::prelude::*;
+
     use super::*;
     use crate::format::{Signed, UnsignedIntoSigned};
-    use opt_ops::prelude::*;
 
     const CT_1: ClockTime = ClockTime::from_nseconds(1);
     const CT_2: ClockTime = ClockTime::from_nseconds(2);
@@ -1085,7 +1340,7 @@ mod tests {
     fn display() {
         let none = Option::<ClockTime>::None;
         let some = Some(45_834_908_569_837 * ClockTime::NSECOND);
-        let lots = ClockTime::from_nseconds(std::u64::MAX - 1);
+        let lots = ClockTime::from_nseconds(u64::MAX - 1);
 
         // Simple
 
@@ -1109,9 +1364,9 @@ mod tests {
             "12:43:54.908569837"
         );
 
-        assert_eq!(format!("{:.0}", lots), "5124095:34:33");
-        assert_eq!(format!("{:.3}", lots), "5124095:34:33.709");
-        assert_eq!(format!("{}", lots), "5124095:34:33.709551614");
+        assert_eq!(format!("{lots:.0}"), "5124095:34:33");
+        assert_eq!(format!("{lots:.3}"), "5124095:34:33.709");
+        assert_eq!(format!("{lots}"), "5124095:34:33.709551614");
 
         // Precision caps at 9
         assert_eq!(
@@ -1122,7 +1377,7 @@ mod tests {
             format!("{:.10}", DisplayableOptClockTime(some)),
             "12:43:54.908569837"
         );
-        assert_eq!(format!("{:.10}", lots), "5124095:34:33.709551614");
+        assert_eq!(format!("{lots:.10}"), "5124095:34:33.709551614");
 
         // Short width
 
@@ -1146,9 +1401,9 @@ mod tests {
             "12:43:54.908569837"
         );
 
-        assert_eq!(format!("{:4.0}", lots), "5124095:34:33");
-        assert_eq!(format!("{:4.3}", lots), "5124095:34:33.709");
-        assert_eq!(format!("{:4}", lots), "5124095:34:33.709551614");
+        assert_eq!(format!("{lots:4.0}"), "5124095:34:33");
+        assert_eq!(format!("{lots:4.3}"), "5124095:34:33.709");
+        assert_eq!(format!("{lots:4}"), "5124095:34:33.709551614");
 
         // Simple padding
 
@@ -1226,15 +1481,15 @@ mod tests {
             " 12:43:54.908569837 "
         );
 
-        assert_eq!(format!("{:>14.0}", lots), " 5124095:34:33");
-        assert_eq!(format!("{:<14.0}", lots), "5124095:34:33 ");
-        assert_eq!(format!("{:^15.0}", lots), " 5124095:34:33 ");
-        assert_eq!(format!("{:>18.3}", lots), " 5124095:34:33.709");
-        assert_eq!(format!("{:<18.3}", lots), "5124095:34:33.709 ");
-        assert_eq!(format!("{:^19.3}", lots), " 5124095:34:33.709 ");
-        assert_eq!(format!("{:>24}", lots), " 5124095:34:33.709551614");
-        assert_eq!(format!("{:<24}", lots), "5124095:34:33.709551614 ");
-        assert_eq!(format!("{:^25}", lots), " 5124095:34:33.709551614 ");
+        assert_eq!(format!("{lots:>14.0}"), " 5124095:34:33");
+        assert_eq!(format!("{lots:<14.0}"), "5124095:34:33 ");
+        assert_eq!(format!("{lots:^15.0}"), " 5124095:34:33 ");
+        assert_eq!(format!("{lots:>18.3}"), " 5124095:34:33.709");
+        assert_eq!(format!("{lots:<18.3}"), "5124095:34:33.709 ");
+        assert_eq!(format!("{lots:^19.3}"), " 5124095:34:33.709 ");
+        assert_eq!(format!("{lots:>24}"), " 5124095:34:33.709551614");
+        assert_eq!(format!("{lots:<24}"), "5124095:34:33.709551614 ");
+        assert_eq!(format!("{lots:^25}"), " 5124095:34:33.709551614 ");
 
         // Padding with sign or zero-extension
 
@@ -1312,15 +1567,15 @@ mod tests {
             "+0012:43:54.908569837"
         );
 
-        assert_eq!(format!("{:+16.0}", lots), "  +5124095:34:33");
-        assert_eq!(format!("{:016.0}", lots), "0005124095:34:33");
-        assert_eq!(format!("{:+016.0}", lots), "+005124095:34:33");
-        assert_eq!(format!("{:+20.3}", lots), "  +5124095:34:33.709");
-        assert_eq!(format!("{:020.3}", lots), "0005124095:34:33.709");
-        assert_eq!(format!("{:+020.3}", lots), "+005124095:34:33.709");
-        assert_eq!(format!("{:+26}", lots), "  +5124095:34:33.709551614");
-        assert_eq!(format!("{:026}", lots), "0005124095:34:33.709551614");
-        assert_eq!(format!("{:+026}", lots), "+005124095:34:33.709551614");
+        assert_eq!(format!("{lots:+16.0}"), "  +5124095:34:33");
+        assert_eq!(format!("{lots:016.0}"), "0005124095:34:33");
+        assert_eq!(format!("{lots:+016.0}"), "+005124095:34:33");
+        assert_eq!(format!("{lots:+20.3}"), "  +5124095:34:33.709");
+        assert_eq!(format!("{lots:020.3}"), "0005124095:34:33.709");
+        assert_eq!(format!("{lots:+020.3}"), "+005124095:34:33.709");
+        assert_eq!(format!("{lots:+26}"), "  +5124095:34:33.709551614");
+        assert_eq!(format!("{lots:026}"), "0005124095:34:33.709551614");
+        assert_eq!(format!("{lots:+026}"), "+005124095:34:33.709551614");
     }
 
     #[test]
@@ -1342,4 +1597,255 @@ mod tests {
     fn attempt_to_build_from_u64max() {
         let _ = ClockTime::from_nseconds(u64::MAX);
     }
+
+    #[test]
+    fn try_into_signed() {
+        let time = crate::Signed::Positive(ClockTime::from_nseconds(0));
+        assert_eq!(i64::try_from(time), Ok(0));
+
+        let time = crate::Signed::Positive(ClockTime::from_nseconds(123));
+        assert_eq!(i64::try_from(time), Ok(123));
+
+        let time = crate::Signed::Positive(ClockTime::from_nseconds(u64::MAX - 1));
+        assert!(i64::try_from(time).is_err());
+
+        let time = crate::Signed::Positive(ClockTime::from_nseconds(u64::MAX >> 1));
+        assert_eq!(i64::MAX as i128, (u64::MAX >> 1) as i128);
+        assert_eq!(i64::try_from(time), Ok(i64::MAX));
+
+        let time = crate::Signed::Negative(ClockTime::from_nseconds(0));
+        assert_eq!(i64::try_from(time), Ok(0));
+
+        let time = crate::Signed::Negative(ClockTime::from_nseconds(123));
+        assert_eq!(i64::try_from(time), Ok(-123));
+
+        let time = crate::Signed::Negative(ClockTime::from_nseconds(u64::MAX - 1));
+        assert!(i64::try_from(time).is_err());
+
+        let time = crate::Signed::Negative(ClockTime::from_nseconds(u64::MAX >> 1));
+        assert_eq!(i64::MIN as i128 + 1, -((u64::MAX >> 1) as i128));
+        assert_eq!(i64::try_from(time), Ok(i64::MIN + 1));
+
+        let time = crate::Signed::Negative(ClockTime::from_nseconds((u64::MAX >> 1) + 1));
+        assert_eq!(i64::MIN as i128, -(((u64::MAX >> 1) + 1) as i128));
+        assert_eq!(i64::try_from(time), Ok(i64::MIN));
+    }
+
+    #[test]
+    fn properties_macro_usage() {
+        use super::ClockTime;
+        use glib::{prelude::*, subclass::prelude::*};
+        use std::cell::Cell;
+
+        #[derive(Default, glib::Properties)]
+        #[properties(wrapper_type = TestObject)]
+        pub struct TestObjectImp {
+            #[property(get, set)]
+            clock_time: Cell<ClockTime>,
+            #[property(get, set)]
+            optional_clock_time: Cell<Option<ClockTime>>,
+        }
+
+        #[glib::object_subclass]
+        impl ObjectSubclass for TestObjectImp {
+            const NAME: &'static str = "GstTestObject";
+            type Type = TestObject;
+        }
+
+        impl ObjectImpl for TestObjectImp {
+            fn properties() -> &'static [glib::ParamSpec] {
+                Self::derived_properties()
+            }
+
+            fn set_property(&self, id: usize, value: &glib::Value, pspec: &glib::ParamSpec) {
+                self.derived_set_property(id, value, pspec);
+            }
+
+            fn property(&self, id: usize, pspec: &glib::ParamSpec) -> glib::Value {
+                self.derived_property(id, pspec)
+            }
+        }
+
+        glib::wrapper! {
+            pub struct TestObject(ObjectSubclass<TestObjectImp>);
+        }
+
+        let obj: TestObject = glib::Object::new();
+
+        assert_eq!(obj.clock_time(), ClockTime::default());
+        obj.set_clock_time(ClockTime::MAX);
+        assert_eq!(obj.clock_time(), ClockTime::MAX);
+
+        assert_eq!(obj.optional_clock_time(), None);
+        obj.set_optional_clock_time(ClockTime::MAX);
+        assert_eq!(obj.optional_clock_time(), Some(ClockTime::MAX));
+    }
+
+    #[test]
+    fn seconds_float() {
+        let res = ClockTime::ZERO;
+        assert_eq!(res.seconds_f32(), 0.0);
+        assert_eq!(res.seconds_f64(), 0.0);
+
+        let res = ClockTime::from_nseconds(2_700_000_000);
+        assert_eq!(res.seconds_f32(), 2.7);
+        assert_eq!(res.seconds_f64(), 2.7);
+
+        let res = ClockTime::MAX;
+        assert_eq!(res.seconds_f32(), 18_446_744_073.709_553);
+        assert_eq!(res.seconds_f64(), 18_446_744_073.709_553);
+    }
+
+    #[test]
+    fn seconds_float_signed() {
+        let pos = Signed::Positive(ClockTime::ZERO);
+        assert_eq!(pos.seconds_f32(), 0.0);
+        assert_eq!(pos.seconds_f64(), 0.0);
+        let neg = Signed::Negative(ClockTime::ZERO);
+        assert_eq!(neg.seconds_f32(), 0.0);
+        assert_eq!(neg.seconds_f64(), 0.0);
+
+        let pos = Signed::Positive(ClockTime::from_nseconds(2_700_000_000));
+        assert_eq!(pos.seconds_f32(), 2.7);
+        assert_eq!(pos.seconds_f64(), 2.7);
+        let neg = Signed::Negative(ClockTime::from_nseconds(2_700_000_000));
+        assert_eq!(neg.seconds_f32(), -2.7);
+        assert_eq!(neg.seconds_f64(), -2.7);
+
+        let pos = Signed::Positive(ClockTime::MAX);
+        assert_eq!(pos.seconds_f32(), 18_446_744_073.709_553);
+        assert_eq!(pos.seconds_f64(), 18_446_744_073.709_553);
+        let neg = Signed::Negative(ClockTime::MAX);
+        assert_eq!(neg.seconds_f32(), -18_446_744_073.709_553);
+        assert_eq!(neg.seconds_f64(), -18_446_744_073.709_553);
+    }
+
+    #[test]
+    fn try_from_seconds_f32() {
+        let res = ClockTime::try_from_seconds_f32(0.0);
+        assert_eq!(res, Ok(ClockTime::ZERO));
+        let res = ClockTime::try_from_seconds_f32(1e-20);
+        assert_eq!(res, Ok(ClockTime::ZERO));
+        let res = ClockTime::try_from_seconds_f32(4.2e-7);
+        assert_eq!(res, Ok(ClockTime::from_nseconds(420)));
+        let res = ClockTime::try_from_seconds_f32(2.7);
+        assert_eq!(res, Ok(ClockTime::from_nseconds(2_700_000_048)));
+        // subnormal float:
+        let res = ClockTime::try_from_seconds_f32(f32::from_bits(1));
+        assert_eq!(res, Ok(ClockTime::ZERO));
+
+        // the conversion uses rounding with tie resolution to even
+        let res = ClockTime::try_from_seconds_f32(0.999e-9);
+        assert_eq!(res, Ok(ClockTime::from_nseconds(1)));
+
+        let res = ClockTime::try_from_seconds_f32(-5.0);
+        assert!(res.is_err());
+        let res = ClockTime::try_from_seconds_f32(f32::NAN);
+        assert!(res.is_err());
+        let res = ClockTime::try_from_seconds_f32(2e19);
+        assert!(res.is_err());
+
+        // this float represents exactly 976562.5e-9
+        let val = f32::from_bits(0x3A80_0000);
+        let res = ClockTime::try_from_seconds_f32(val);
+        assert_eq!(res, Ok(ClockTime::from_nseconds(976_562)));
+
+        // this float represents exactly 2929687.5e-9
+        let val = f32::from_bits(0x3B40_0000);
+        let res = ClockTime::try_from_seconds_f32(val);
+        assert_eq!(res, Ok(ClockTime::from_nseconds(2_929_688)));
+
+        // this float represents exactly 1.000_976_562_5
+        let val = f32::from_bits(0x3F802000);
+        let res = ClockTime::try_from_seconds_f32(val);
+        assert_eq!(res, Ok(ClockTime::from_nseconds(1_000_976_562)));
+
+        // this float represents exactly 1.002_929_687_5
+        let val = f32::from_bits(0x3F806000);
+        let res = ClockTime::try_from_seconds_f32(val);
+        assert_eq!(res, Ok(ClockTime::from_nseconds(1_002_929_688)));
+    }
+
+    #[test]
+    fn try_from_seconds_f64() {
+        let res = ClockTime::try_from_seconds_f64(0.0);
+        assert_eq!(res, Ok(ClockTime::ZERO));
+        let res = ClockTime::try_from_seconds_f64(1e-20);
+        assert_eq!(res, Ok(ClockTime::ZERO));
+        let res = ClockTime::try_from_seconds_f64(4.2e-7);
+        assert_eq!(res, Ok(ClockTime::from_nseconds(420)));
+        let res = ClockTime::try_from_seconds_f64(2.7);
+        assert_eq!(res, Ok(ClockTime::from_nseconds(2_700_000_000)));
+        // subnormal float:
+        let res = ClockTime::try_from_seconds_f64(f64::from_bits(1));
+        assert_eq!(res, Ok(ClockTime::ZERO));
+
+        // the conversion uses rounding with tie resolution to even
+        let res = ClockTime::try_from_seconds_f64(0.999e-9);
+        assert_eq!(res, Ok(ClockTime::from_nseconds(1)));
+        let res = ClockTime::try_from_seconds_f64(0.999_999_999_499);
+        assert_eq!(res, Ok(ClockTime::from_nseconds(999_999_999)));
+        let res = ClockTime::try_from_seconds_f64(0.999_999_999_501);
+        assert_eq!(res, Ok(ClockTime::from_seconds(1)));
+        let res = ClockTime::try_from_seconds_f64(42.999_999_999_499);
+        assert_eq!(res, Ok(ClockTime::from_nseconds(42_999_999_999)));
+        let res = ClockTime::try_from_seconds_f64(42.999_999_999_501);
+        assert_eq!(res, Ok(ClockTime::from_seconds(43)));
+
+        let res = ClockTime::try_from_seconds_f64(-5.0);
+        assert!(res.is_err());
+        let res = ClockTime::try_from_seconds_f64(f64::NAN);
+        assert!(res.is_err());
+        let res = ClockTime::try_from_seconds_f64(2e19);
+        assert!(res.is_err());
+
+        // this float represents exactly 976562.5e-9
+        let val = f64::from_bits(0x3F50_0000_0000_0000);
+        let res = ClockTime::try_from_seconds_f64(val);
+        assert_eq!(res, Ok(ClockTime::from_nseconds(976_562)));
+
+        // this float represents exactly 2929687.5e-9
+        let val = f64::from_bits(0x3F68_0000_0000_0000);
+        let res = ClockTime::try_from_seconds_f64(val);
+        assert_eq!(res, Ok(ClockTime::from_nseconds(2_929_688)));
+
+        // this float represents exactly 1.000_976_562_5
+        let val = f64::from_bits(0x3FF0_0400_0000_0000);
+        let res = ClockTime::try_from_seconds_f64(val);
+        assert_eq!(res, Ok(ClockTime::from_nseconds(1_000_976_562)));
+
+        // this float represents exactly 1.002_929_687_5
+        let val = f64::from_bits(0x3FF0_0C00_0000_0000);
+        let res = ClockTime::try_from_seconds_f64(val);
+        assert_eq!(res, Ok(ClockTime::from_nseconds(1_002_929_688)));
+    }
+
+    #[test]
+    fn try_from_seconds_f32_signed() {
+        let pos = Signed::<ClockTime>::from_seconds_f32(5.0);
+        assert!(pos.is_positive());
+
+        let neg = Signed::<ClockTime>::from_seconds_f32(-5.0);
+        assert!(neg.is_negative());
+    }
+
+    #[test]
+    fn try_from_seconds_f64_signed() {
+        let pos = Signed::<ClockTime>::from_seconds_f64(5.0);
+        assert!(pos.is_positive());
+
+        let neg = Signed::<ClockTime>::from_seconds_f64(-5.0);
+        assert!(neg.is_negative());
+    }
+
+    #[test]
+    fn absdiff() {
+        let t1 = ClockTime::from_seconds(10);
+        let t2 = ClockTime::from_seconds(4);
+
+        let d = ClockTime::from_seconds(6);
+
+        assert_eq!(t1.absdiff(t2), d);
+        assert_eq!(t2.absdiff(t1), d);
+    }
 }
