@@ -126,7 +126,7 @@ macro_rules! mini_object_wrapper (
 
         impl $crate::glib::translate::IntoGlibPtr<*mut $ffi_name> for $name {
             #[inline]
-            unsafe fn into_glib_ptr(self) -> *mut $ffi_name {
+            fn into_glib_ptr(self) -> *mut $ffi_name {
                 let s = std::mem::ManuallyDrop::new(self);
                 s.as_mut_ptr()
             }
@@ -209,7 +209,7 @@ macro_rules! mini_object_wrapper (
             type Storage = std::marker::PhantomData<&'a mut Self>;
 
             #[inline]
-            fn to_glib_none_mut(&'a mut self) -> $crate::glib::translate::StashMut<*mut $ffi_name, Self> {
+            fn to_glib_none_mut(&'_ mut self) -> $crate::glib::translate::StashMut<'_, *mut $ffi_name, Self> {
                 self.make_mut();
                 $crate::glib::translate::StashMut(self.as_mut_ptr(), std::marker::PhantomData)
             }
@@ -647,6 +647,24 @@ impl MiniObject {
             Err(self)
         }
     }
+
+    #[inline]
+    pub fn downcast_ref<T: IsMiniObject + StaticType>(&self) -> Option<&T> {
+        if self.type_().is_a(T::static_type()) {
+            unsafe { Some(&*(self as *const Self as *const T)) }
+        } else {
+            None
+        }
+    }
+
+    #[inline]
+    pub fn downcast_mut<T: IsMiniObject + StaticType>(&mut self) -> Option<&mut T> {
+        if self.type_().is_a(T::static_type()) {
+            unsafe { Some(&mut *(self as *mut Self as *mut T)) }
+        } else {
+            None
+        }
+    }
 }
 
 impl fmt::Debug for MiniObject {
