@@ -5,19 +5,11 @@ use glib::{
     translate::{from_glib, FromGlibPtrFull, IntoGlib, ToGlibPtr},
 };
 
-use crate::{PluginFeature, Rank};
+use crate::{ffi, PluginFeature, Rank};
 
-pub trait PluginFeatureExtManual: Sized + 'static {
+pub trait PluginFeatureExtManual: IsA<PluginFeature> + 'static {
     #[doc(alias = "get_rank")]
     #[doc(alias = "gst_plugin_feature_get_rank")]
-    fn rank(&self) -> Rank;
-    #[doc(alias = "gst_plugin_feature_set_rank")]
-    fn set_rank(&self, rank: Rank);
-    #[doc(alias = "gst_plugin_feature_load")]
-    fn load(&self) -> Result<Self, glib::BoolError>;
-}
-
-impl<O: IsA<PluginFeature>> PluginFeatureExtManual for O {
     fn rank(&self) -> Rank {
         unsafe {
             let rank = ffi::gst_plugin_feature_get_rank(self.as_ref().to_glib_none().0);
@@ -25,6 +17,7 @@ impl<O: IsA<PluginFeature>> PluginFeatureExtManual for O {
         }
     }
 
+    #[doc(alias = "gst_plugin_feature_set_rank")]
     fn set_rank(&self, rank: Rank) {
         unsafe {
             ffi::gst_plugin_feature_set_rank(
@@ -34,6 +27,7 @@ impl<O: IsA<PluginFeature>> PluginFeatureExtManual for O {
         }
     }
 
+    #[doc(alias = "gst_plugin_feature_load")]
     fn load(&self) -> Result<Self, glib::BoolError> {
         unsafe {
             let loaded = Option::<PluginFeature>::from_glib_full(ffi::gst_plugin_feature_load(
@@ -45,6 +39,8 @@ impl<O: IsA<PluginFeature>> PluginFeatureExtManual for O {
     }
 }
 
+impl<O: IsA<PluginFeature>> PluginFeatureExtManual for O {}
+
 #[cfg(test)]
 mod tests {
     use super::*;
