@@ -6,7 +6,7 @@ use glib::{subclass::prelude::*, translate::*};
 
 use super::prelude::*;
 use crate::{
-    prelude::*, Element, Event, PadTemplate, QueryRef, StateChange, StateChangeError,
+    ffi, prelude::*, Element, Event, PadTemplate, QueryRef, StateChange, StateChangeError,
     StateChangeReturn, StateChangeSuccess,
 };
 
@@ -67,7 +67,7 @@ impl ElementMetadata {
     }
 }
 
-pub trait ElementImpl: ElementImplExt + GstObjectImpl + Send + Sync {
+pub trait ElementImpl: GstObjectImpl + ObjectSubclass<Type: IsA<Element>> {
     fn metadata() -> Option<&'static ElementMetadata> {
         None
     }
@@ -121,12 +121,7 @@ pub trait ElementImpl: ElementImplExt + GstObjectImpl + Send + Sync {
     }
 }
 
-mod sealed {
-    pub trait Sealed {}
-    impl<T: super::ElementImplExt> Sealed for T {}
-}
-
-pub trait ElementImplExt: sealed::Sealed + ObjectSubclass {
+pub trait ElementImplExt: ElementImpl {
     fn parent_change_state(
         &self,
         transition: StateChange,
@@ -161,7 +156,7 @@ pub trait ElementImplExt: sealed::Sealed + ObjectSubclass {
                     from_glib_none(f(
                         self.obj().unsafe_cast_ref::<Element>().to_glib_none().0,
                         templ.to_glib_none().0,
-                        name.to_glib_full(),
+                        name.to_glib_none().0,
                         caps.to_glib_none().0,
                     ))
                 })
@@ -286,8 +281,16 @@ pub trait ElementImplExt: sealed::Sealed + ObjectSubclass {
 
     #[inline(never)]
     fn panicked(&self) -> &atomic::AtomicBool {
-        self.instance_data::<atomic::AtomicBool>(crate::Element::static_type())
-            .expect("instance not initialized correctly")
+        #[cfg(panic = "abort")]
+        {
+            static DUMMY: atomic::AtomicBool = atomic::AtomicBool::new(false);
+            &DUMMY
+        }
+        #[cfg(not(panic = "abort"))]
+        {
+            self.instance_data::<atomic::AtomicBool>(crate::Element::static_type())
+                .expect("instance not initialized correctly")
+        }
     }
 
     fn catch_panic<R, F: FnOnce(&Self) -> R, G: FnOnce() -> R>(&self, fallback: G, f: F) -> R {
@@ -408,6 +411,7 @@ unsafe impl<T: ElementImpl> IsSubclassable<T> for Element {
     fn instance_init(instance: &mut glib::subclass::InitializingObject<T>) {
         Self::parent_instance_init::<T>(instance);
 
+        #[cfg(not(panic = "abort"))]
         instance.set_instance_data(Self::static_type(), atomic::AtomicBool::new(false));
     }
 }
@@ -429,7 +433,10 @@ unsafe extern "C" fn element_change_state<T: ElementImpl>(
         _ => StateChangeReturn::Failure,
     };
 
-    panic_to_error!(imp, fallback, { imp.change_state(transition).into() }).into_glib()
+    panic_to_error!(imp, fallback, {
+        StateChangeReturn::from(imp.change_state(transition))
+    })
+    .into_glib()
 }
 
 unsafe extern "C" fn element_request_new_pad<T: ElementImpl>(
@@ -551,7 +558,7 @@ unsafe extern "C" fn element_post_message<T: ElementImpl>(
 
 #[cfg(test)]
 mod tests {
-    use std::sync::atomic;
+    use std::sync::{atomic, Arc, Mutex, OnceLock};
 
     use super::*;
     use crate::ElementFactory;
@@ -564,6 +571,7 @@ mod tests {
             pub(super) sinkpad: crate::Pad,
             pub(super) n_buffers: atomic::AtomicU32,
             pub(super) reached_playing: atomic::AtomicBool,
+            pub(super) array: Arc<Mutex<Vec<String>>>,
         }
 
         impl TestElement {
@@ -646,6 +654,10 @@ mod tests {
                 Self {
                     n_buffers: atomic::AtomicU32::new(0),
                     reached_playing: atomic::AtomicBool::new(false),
+                    array: Arc::new(Mutex::new(vec![
+                        "default0".to_string(),
+                        "default1".to_string(),
+                    ])),
                     srcpad,
                     sinkpad,
                 }
@@ -660,6 +672,30 @@ mod tests {
                 element.add_pad(&self.sinkpad).unwrap();
                 element.add_pad(&self.srcpad).unwrap();
             }
+
+            fn properties() -> &'static [glib::ParamSpec] {
+                static PROPERTIES: OnceLock<Vec<glib::ParamSpec>> = OnceLock::new();
+                PROPERTIES.get_or_init(|| vec![crate::ParamSpecArray::builder("array").build()])
+            }
+
+            fn set_property(&self, _id: usize, value: &glib::Value, pspec: &glib::ParamSpec) {
+                match pspec.name() {
+                    "array" => {
+                        let value = value.get::<crate::Array>().unwrap();
+                        let mut array = self.array.lock().unwrap();
+                        array.clear();
+                        array.extend(value.iter().map(|v| v.get().unwrap()));
+                    }
+                    _ => unimplemented!(),
+                }
+            }
+
+            fn property(&self, _id: usize, pspec: &glib::ParamSpec) -> glib::Value {
+                match pspec.name() {
+                    "array" => crate::Array::new(&*self.array.lock().unwrap()).to_value(),
+                    _ => unimplemented!(),
+                }
+            }
         }
 
         impl GstObjectImpl for TestElement {}
@@ -729,9 +765,40 @@ mod tests {
         }
     }
 
+    fn plugin_init(plugin: &crate::Plugin) -> Result<(), glib::BoolError> {
+        crate::Element::register(
+            Some(plugin),
+            "testelement",
+            crate::Rank::MARGINAL,
+            TestElement::static_type(),
+        )
+    }
+
+    crate::plugin_define!(
+        rssubclasstestelem,
+        env!("CARGO_PKG_DESCRIPTION"),
+        plugin_init,
+        env!("CARGO_PKG_VERSION"),
+        "MPL-2.0",
+        env!("CARGO_PKG_NAME"),
+        env!("CARGO_PKG_NAME"),
+        env!("CARGO_PKG_REPOSITORY"),
+        "1970-01-01"
+    );
+
+    fn init() {
+        use std::sync::Once;
+        static INIT: Once = Once::new();
+
+        INIT.call_once(|| {
+            crate::init().unwrap();
+            plugin_register_static().expect("gstreamer subclass element test");
+        });
+    }
+
     #[test]
     fn test_element_subclass() {
-        crate::init().unwrap();
+        init();
 
         let element = TestElement::new(Some("test"));
 
@@ -766,4 +833,47 @@ mod tests {
         assert_eq!(imp.n_buffers.load(atomic::Ordering::SeqCst), 100);
         assert!(imp.reached_playing.load(atomic::Ordering::SeqCst));
     }
+
+    #[test]
+    fn property_from_iter_if_not_empty() {
+        init();
+
+        let elem = crate::ElementFactory::make("testelement").build().unwrap();
+        assert!(elem
+            .property::<crate::Array>("array")
+            .iter()
+            .map(|val| val.get::<&str>().unwrap())
+            .eq(["default0", "default1"]));
+
+        let elem = crate::ElementFactory::make("testelement")
+            .property_from_iter::<crate::Array, _>("array", ["value0", "value1"])
+            .build()
+            .unwrap();
+        assert!(elem
+            .property::<crate::Array>("array")
+            .iter()
+            .map(|val| val.get::<&str>().unwrap())
+            .eq(["value0", "value1"]));
+
+        let array = Vec::<String>::new();
+        let elem = crate::ElementFactory::make("testelement")
+            .property_if_not_empty::<crate::Array, _>("array", &array)
+            .build()
+            .unwrap();
+        assert!(elem
+            .property::<crate::Array>("array")
+            .iter()
+            .map(|val| val.get::<&str>().unwrap())
+            .eq(["default0", "default1"]));
+
+        let elem = crate::ElementFactory::make("testelement")
+            .property_if_not_empty::<crate::Array, _>("array", ["value0", "value1"])
+            .build()
+            .unwrap();
+        assert!(elem
+            .property::<crate::Array>("array")
+            .iter()
+            .map(|val| val.get::<&str>().unwrap())
+            .eq(["value0", "value1"]));
+    }
 }
