@@ -1,13 +1,11 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use serde::de::{Deserialize, Deserializer};
-use serde::ser::{Serialize, SerializeStruct, Serializer};
+use serde::{
+    de::{Deserialize, Deserializer},
+    ser::{Serialize, SerializeStruct, Serializer},
+};
 
-use crate::toc::*;
-use crate::TagList;
-use crate::TocEntryType;
-use crate::TocLoopType;
-use crate::TocScope;
+use crate::{toc::*, TagList, TocEntryType, TocLoopType, TocScope};
 
 impl Serialize for TocRef {
     fn serialize<S: Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
@@ -117,12 +115,7 @@ impl<'de> Deserialize<'de> for TocEntry {
 
 #[cfg(test)]
 mod tests {
-    use crate::tags::Title;
-    use crate::toc::*;
-    use crate::TagList;
-    use crate::TagMergeMode;
-    use crate::TocEntryType;
-    use crate::TocScope;
+    use crate::{tags::Title, toc::*, TagList, TagMergeMode, TocEntryType, TocScope};
 
     #[test]
     fn test_serialize() {
@@ -208,14 +201,14 @@ mod tests {
                 "            uid: \"edition\",",
                 "            start_stop: Some((0, 15)),",
                 "            tags: None,",
-                "            loop: Some((None, 0)),",
+                "            loop: Some((r#None, 0)),",
                 "            sub_entries: [",
                 "                (",
                 "                    entry_type: Chapter,",
                 "                    uid: \"chapter1\",",
                 "                    start_stop: Some((0, 10)),",
                 "                    tags: None,",
-                "                    loop: Some((None, 0)),",
+                "                    loop: Some((r#None, 0)),",
                 "                    sub_entries: [",
                 "                        (",
                 "                            entry_type: Chapter,",
@@ -229,7 +222,7 @@ mod tests {
                 "                                    ]),",
                 "                                ],",
                 "                            )),",
-                "                            loop: Some((None, 0)),",
+                "                            loop: Some((r#None, 0)),",
                 "                            sub_entries: [],",
                 "                        ),",
                 "                        (",
@@ -244,7 +237,7 @@ mod tests {
                 "                                    ]),",
                 "                                ],",
                 "                            )),",
-                "                            loop: Some((None, 0)),",
+                "                            loop: Some((r#None, 0)),",
                 "                            sub_entries: [],",
                 "                        ),",
                 "                    ],",
@@ -261,7 +254,7 @@ mod tests {
                 "                            ]),",
                 "                        ],",
                 "                    )),",
-                "                    loop: Some((None, 0)),",
+                "                    loop: Some((r#None, 0)),",
                 "                    sub_entries: [],",
                 "                ),",
                 "            ],",
